@@ -53,33 +53,21 @@ from epoptes.common import ltsconf
 from epoptes.common import config
 from epoptes.common.constants import *
 from epoptes.core import wol
-
-
-
-## FIXME: This is dummy
-def _startswith(string, pattern):
-    if pattern is None:
-        return True
-    return string.startswith(pattern)
-
-# Warn users to update their chroots if they have a lower epoptes-client version 
-# than this
-COMPATIBILITY_VERSION = [0, 1]
+from epoptes.core import structs
 
 
 class EpoptesGui(object):
     
-    def __init__(self, conf, host_filter):
-        self.conf = conf
-        self.host_filter = host_filter
-        self.ltsConf = ltsconf.ltsConf()
+    def __init__(self):
         self.c = commands.commands()
         self.shownCompatibilityWarning = False 
-        self.added = False
         self.vncserver = None
         self.vncviewer = None
         self.scrWidth = 100
         self.scrHeight = 75
+        self.current_macs = subprocess.Popen(['sh', '-c', 
+        """ip -oneline -family inet link show | sed -n '/.*ether[[:space:]]*\\([[:xdigit:]:]*\).*/{s//\\1/;y/abcdef-/ABCDEF:/;p;}'"""], 
+                                             stdout=subprocess.PIPE).stdout.read().split()
         if os.getuid() != 0:
             if 'thumbnails_width' in config.user:
                 self.scrWidth = config.user['thumbnails_width']
@@ -90,58 +78,69 @@ class EpoptesGui(object):
         self.fat = gtk.gdk.pixbuf_new_from_file('images/fat.svg')
         self.standalone = gtk.gdk.pixbuf_new_from_file('images/standalone.svg')
         self.imagetypes = {'thin' : self.thin, 'fat' : self.fat,
-            'standalone' : self.standalone, 'server' : self.standalone}
-        self.addStockImage('offline', 'images/off.png')
-        self.addStockImage('online', 'images/on.png')
-        self.addStockImage('users', 'images/usersgrp.png')
-        self.addStockImage('system', 'images/systemgrp.png')
+            'standalone' : self.standalone, 'server' : self.standalone, 'offline' : self.offline}
         
         self.wTree = gtk.Builder()
         self.wTree.add_from_file('epoptes.ui')
-
+        
         # Connect glade handlers with the callback functions
         self.wTree.connect_signals(self)
         self.get = lambda obj: self.wTree.get_object(obj)
-
+        
+        self.gstore = gtk.ListStore(str, object, bool)
+        
+        self.gtree = self.get("groups_tree")
+        self.gtree.set_model(self.gstore)
+        self.gtree.get_selection().connect("changed", self.on_group_selection_changed)
+        
         # Enable this when the scrollbar for the screenshot size is available
         #self.get('iconsSizeAdjustment').set_value(self.scrWidth)
 
         self.mainwin = self.get('mainwindow')
         
-        self.cstore = gtk.ListStore(str, str, str, str, str, str, 
-            gtk.gdk.Pixbuf, str, str)
+        self.cstore = gtk.ListStore(str, gtk.gdk.Pixbuf, object, str)
         self.cview = self.get('clientsview')
-        self.cView_order=(1, 0)
-        self.refresh()
+        self.cView_order = (1, 0)
+        self.set_cView(*self.cView_order)
         
-        self.cfilter = self.cstore.filter_new()
-        self.cfilter.set_visible_func(self.setVisibleClients)
-        self.csort = gtk.TreeModelSort(self.cfilter)
-        self.cview.set_model(self.csort)
+        self.cview.set_model(self.cstore)
+        self.cview.set_pixbuf_column(C_PIXBUF)
+        self.cview.set_text_column(C_LABEL)
         
+        self.cstore.set_sort_column_id(C_LABEL, gtk.SORT_ASCENDING)
+        self.on_clients_selection_changed()
         
-        self.cview.set_text_column(8)
-        self.cview.set_pixbuf_column(6)
-        self.csort.set_sort_column_id(8, gtk.SORT_ASCENDING)
-        self.setClientMenuSensitivity()
-        self.toggleRealNameColumn()
+        self.cview.enable_model_drag_source(gtk.gdk.BUTTON1_MASK, [("add", gtk.TARGET_SAME_APP, 0)], gtk.gdk.ACTION_COPY)
+        self.gtree.enable_model_drag_dest([("add", gtk.TARGET_SAME_APP, 0)], gtk.gdk.ACTION_COPY)
+        
+        self.default_group = structs.Group('<b>'+_('Detected clients')+'</b>')
+        default_iter = self.gstore.append([self.default_group.name, self.default_group, False])
+        self.default_group.ref = gtk.TreeRowReference(self.gstore, self.gstore.get_path(default_iter))
+        self.gtree.get_selection().select_path(self.default_group.ref.get_path())
+        
+        saved_clients, groups = config.read_groups(os.path.expanduser('~/.config/epoptes/groups.json'))
+        for grp in groups:
+            self.gstore.append([grp.name, grp, True])
+        
+        self.fillIconView(self.getSelectedGroup()[1])
 
     #################################################################
     #                       Callback functions                      #
     #################################################################
-
-    def refresh(self, widget=None):
-        """
-        Refresh clicked
-
-        Refresh main dialog by re-loading panels containing
-        lists of clients, groups and users connected.
-        """
-        self.ltsConf.parse()
-        self.loadClients()
-        self.set_cView(self.cView_order[0], self.cView_order[1])
-        self.setClientMenuSensitivity()
-
+    def on_gtree_drag_drop(self, wid, context, x, y, time):
+        dest = self.gtree.get_dest_row_at_pos(x, y)
+        if dest is not None:
+            path, pos = dest
+            if pos in (gtk.TREE_VIEW_DROP_INTO_OR_BEFORE, gtk.TREE_VIEW_DROP_INTO_OR_AFTER):
+                group = self.gstore[path][G_INSTANCE]
+                if not group is self.default_group:
+                    for cln in self.getSelectedClients():
+                        cln = cln[C_INSTANCE]
+                        if not group.has_client(cln):
+                            group.add_client(cln)
+    
+        context.finish(True, False, time)
+        return True
 
     def on_mainwin_destroy(self, widget):
         """
@@ -149,49 +148,41 @@ class EpoptesGui(object):
 
         Close main window
         """
+        self.gstore.remove(self.gstore.get_iter(self.default_group.ref.get_path()))
+        config.save_groups(os.path.expanduser('~/.config/epoptes/groups.json'), self.gstore)
         if not self.vncserver is None:
             self.vncserver.kill()
         if not self.vncviewer is None:
             self.vncviewer.kill()
         reactor.stop()
-
-
-    def toggleRealNameColumn(self, widget=None):
-        """Show/hide the real names of the users instead of the username"""
+            
+    def toggleRealNames(self, widget=None):
+        """Show/hide the real names of the users instead of the usernames"""
         pass # Implement me
 
     def wake_on_lan(self, widget):
-        """
-        For clients selected in clients' tree power them on using their
-        MAC addresses
-        """
+        """Boot the selected computers with WOL"""
         for client in self.getSelectedClients():
             # Make sure that only offline computers will be sent to wol
-            if client[C_SESSION_HANDLE] == '' and client[C_SYSTEM_HANDLE] == '':
-                wol.wake_on_lan(client[C_MAC])
+            client = client[C_INSTANCE]
+            if client.is_offline():
+                wol.wake_on_lan(client.mac)
 
     def poweroff(self, widget):
-        """
-        For clients selected in clients' tree, power them off
-        """
+        """Shut down the selected clients."""
         self.execOnSelectedClients(self.c.POWEROFF, root="auto",
             warn=self.c.POWEROFF_WARN)
 
     def reboot(self, widget):
-        """
-        For clients selected in clients' tree, reboot them
-        """
+        """Reboot the selected clients."""
         # FIXME: (Not) waiting on purpose to cause some delay to avoid 
         # any power strain.
         self.execOnSelectedClients(self.c.REBOOT, root="auto",
             warn=self.c.REBOOT_WARN)
 
     def logout(self, widget):
-        """
-        For clients selected in clients' tree, log them off
-        """
-        self.execOnSelectedClients(self.c.LOGOUT,
-            warn=self.c.LOGOUT_WARN)
+        """Log off the users of the selected clients."""
+        self.execOnSelectedClients(self.c.LOGOUT, warn=self.c.LOGOUT_WARN)
 
 
     def reverseConnection(self, widget, path, view_column, cmd):
@@ -203,19 +194,18 @@ class EpoptesGui(object):
         self.execOnSelectedClients(self.c.EXEC + cmd)
 
 
-    def assistStudent(self, widget, path=None, view_column=None):
-        self.reverseConnection(widget, path, view_column, 
+    def assistUser(self, widget, path=None, view_column=None):
+        self.reverseConnection(widget, path, view_column,
             'x11vnc -noshm -24to32 -connect_or_exit $SERVER')
 
 
-    def monitorStudent(self, widget, path=None, view_column=None):
+    def monitorUser(self, widget, path=None, view_column=None):
         self.reverseConnection(widget, path, view_column,
             'x11vnc -noshm -24to32 -viewonly -connect_or_exit $SERVER')
 
 
     def findUnusedPort(self, base=None):
-        """Find an unused port, optionally starting from "base".
-        """
+        """Find an unused port, optionally starting from "base"."""
         import socket
 
         s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
@@ -237,7 +227,7 @@ class EpoptesGui(object):
             return None
 
 
-    def broadcastTeacher(self, widget):
+    def broadcastScreen(self, widget):
         if self.vncserver is None:
             self.vncport = self.findUnusedPort()
             # TODO: use a password instead of -allow
@@ -255,8 +245,7 @@ class EpoptesGui(object):
         # Unfortunately, dbus-daemon doesn't contain DBUS_SESSION_BUS_ADDRESS.
         self.execOnSelectedClients("""
 test -n "$EPOPTES_VNCVIEWER_PID" && kill $EPOPTES_VNCVIEWER_PID
-p=$(pgrep -nx 'ldm|kdm_greet|lxdm-greeter-gt|dbus-daemon')
-export $(tr '\\0' '\\n' < /proc/$p/environ | egrep '^DISPLAY=|^XAUTHORITY=')
+export $(./get-display)
 xset dpms force on
 sleep 0.$((`hexdump -e '"%%d"' -n 2 /dev/urandom` %% 50 + 50))
 EPOPTES_VNCVIEWER_PID=$(./execute xvnc4viewer -Shared -ViewOnly -FullScreen \
@@ -274,7 +263,7 @@ unset EPOPTES_VNCVIEWER_PID""", self.cstore, None, True)
             self.vncserver = None
 
 
-    ## FIXME FIXME: Should we allow teacher to run whatever command in clients?
+    ## FIXME FIXME: Should we allow for running arbitrary commands in clients?
     def execDialog(self, widget):
         cmd = startExecuteCmdDlg()
         # If Cancel or Close were clicked
@@ -289,7 +278,7 @@ unset EPOPTES_VNCVIEWER_PID""", self.cstore, None, True)
     ## FIXME FIXME: Don't use zenity...
     def sendMessageDialog(self, widget):
         cmd = startSendMessageDlg()
-        if cmd != "":# Command is 'valid', execute on selected clients 
+        if cmd != "": # Command is 'valid', execute on selected clients 
             as_root = False
             self.execOnSelectedClients(self.c.EXEC + cmd, root=as_root)
     
@@ -302,8 +291,7 @@ unset EPOPTES_VNCVIEWER_PID""", self.cstore, None, True)
 
     def remoteRootTerminal(self, widget):
         self.execOnSelectedClients("""
-p=$(pgrep -nx 'ldm|kdm_greet|lxdm-greeter-gt|dbus-daemon')
-export $(tr '\\0' '\\n' < /proc/$p/environ | egrep '^DISPLAY=|^XAUTHORITY=')
+export $(./get-display)
 ./execute xterm -e bash -l""", root=True)
     ## END_FIXUS
 
@@ -312,15 +300,15 @@ export $(tr '\\0' '\\n' < /proc/$p/environ | egrep '^DISPLAY=|^XAUTHORITY=')
         """
         Lock screen for all clients selected
         """
-        self.execOnSelectedClients('test -n "$EPOPTES_LOCK_SCREEN_PID" && kill ' +\
-            '"$EPOPTES_LOCK_SCREEN_PID"; EPOPTES_LOCK_SCREEN_PID=$(./execute '+\
+        self.execOnSelectedClients('test -n "$EPOPTES_LOCK_SCREEN_PID" && kill ' + \
+            '"$EPOPTES_LOCK_SCREEN_PID"; EPOPTES_LOCK_SCREEN_PID=$(./execute ' + \
             './lock-screen)')
 
     def unlockScreen(self, widget):
         """
         Unlock screen for all clients selected
         """
-        self.execOnSelectedClients('''test -n "$EPOPTES_LOCK_SCREEN_PID" && '''+\
+        self.execOnSelectedClients('''test -n "$EPOPTES_LOCK_SCREEN_PID" && ''' + \
             '''kill "$EPOPTES_LOCK_SCREEN_PID"; unset EPOPTES_LOCK_SCREEN_PID''')
 
     # FIXME: Find something better
@@ -336,9 +324,54 @@ export $(tr '\\0' '\\n' < /proc/$p/environ | egrep '^DISPLAY=|^XAUTHORITY=')
         """
         self.execOnSelectedClients(self.c.EXEC_AMIXER + 'unmute', root=True)
     
+    def on_remove_from_group_clicked(self, widget):
+        clients = self.getSelectedClients()
+        group = self.getSelectedGroup()[1]
+        
+        if self.warnDlgPopup(_('Are you sure you want to remove the selected client(s) from group "%s"?') %group.name):
+            for client in clients:
+                group.remove_client(client[C_INSTANCE])
+            self.fillIconView(self.getSelectedGroup()[1])
+    
+    def set_move_group_sensitivity(self):
+        selected = self.getSelectedGroup()
+        selected_path = self.gstore.get_path(selected[0])[0]
+        blocker = not selected[1] is self.default_group
+        self.get('move_group_up').set_sensitive(blocker and selected_path > 1)
+        self.get('move_group_down').set_sensitive(blocker and selected_path < len(self.gstore)-1)
+    
+    def on_move_group_down_clicked(self, widget):
+        selected_group_iter = self.getSelectedGroup()[0]
+        self.gstore.swap(selected_group_iter, self.gstore.iter_next(selected_group_iter))
+        self.set_move_group_sensitivity()
+    
+    def on_move_group_up_clicked(self, widget):
+        selected_group_iter = self.getSelectedGroup()[0]
+        previous_iter = self.gstore.get_iter(self.gstore.get_path(selected_group_iter)[0]-1)
+        self.gstore.swap(selected_group_iter, previous_iter)
+        self.set_move_group_sensitivity()
+        
+    def on_remove_group_clicked(self, widget):
+        group_iter = self.getSelectedGroup()[0]
+        group = self.gstore[group_iter][G_INSTANCE]
+        
+        if self.warnDlgPopup(_('Are you sure you want to remove group "%s"?') % group.name):
+            self.gstore.remove(group_iter)
+            
+    def on_add_group_clicked(self, widget):
+        new_group = structs.Group()
+        iter = self.gstore.append([new_group.name, new_group, True])
+        # Edit the name of the newly created group
+        self.gtree.set_cursor(self.gstore.get_path(iter), self.get('group_name_column'), True)
+    
+    def on_group_renamed(self, widget, path, new_name):
+        self.gstore[path][G_LABEL] = new_name
+        self.gstore[path][G_INSTANCE].name = new_name
+        
     #FIXME: Remove the second parameter, find a better way
     def on_tb_client_properties_clicked(self, widget=None):
         ClientInformation(self.getSelectedClients(), self.daemon.command).run()
+        self.setLabel(self.getSelectedClients()[0])
     
     def on_mi_remote_assistance_activate(self, widget=None):
         RemoteAssistance().run()
@@ -358,35 +391,19 @@ export $(tr '\\0' '\\n' < /proc/$p/environ | egrep '^DISPLAY=|^XAUTHORITY=')
     def on_cViewU_toggled(self, mitem):
         self.set_cView(0, -1)
     
-    def set_cView(self, user_pos=-1, host_pos=0):
+    def set_cView(self, user_pos= -1, name_pos=0):
         # Save the order so all new clients get the selected format
-        self.cView_order = (user_pos, host_pos)
-        
-        host = lambda x: self.cstore[x][C_HOSTNAME]
-        user = lambda x: self.cstore[x][C_USER]
-        for i in range(len(self.cstore)):
-            self.cstore[i][C_VIEW_STYLE] = ''
-            if user(i) == '' or user_pos == -1:
-                self.cstore[i][C_VIEW_STYLE] = host(i)
-            else:
-                if user_pos == 0:
-                    self.cstore[i][C_VIEW_STYLE] = user(i)
-                    if host_pos == 1:
-                        self.cstore[i][C_VIEW_STYLE] += " (%s)" %host(i)
-                elif host_pos == 0:
-                    self.cstore[i][C_VIEW_STYLE] = host(i)
-                    if user_pos == 1:
-                        self.cstore[i][C_VIEW_STYLE] += " (%s)" %user(i)
-    
-    
+        self.cView_order = (user_pos, name_pos)
+        for row in self.cstore:            
+            self.setLabel(row)
     
     def connected(self, daemon):
         self.daemon = daemon
         daemon.enumerateClients().addCallback(lambda h: self.amp_gotClients(h))
 
     # AMP callbacks
-
     def amp_clientConnected(self, handle):
+        print "New connection from", handle
         self.daemon.command(handle, u"""
             VERSION=$(dpkg-query -W -f '${Version}' epoptes-client 2>/dev/null)
             VERSION=${VERSION:-0.1}
@@ -394,43 +411,32 @@ export $(tr '\\0' '\\n' < /proc/$p/environ | egrep '^DISPLAY=|^XAUTHORITY=')
             """).addCallback(lambda r: self.addClient(handle, r))
 
     def amp_clientDisconnected(self, handle):
-        for client in self.cstore:
-            if client[C_SYSTEM_HANDLE] == handle:
-                shutdownNotify(client[C_HOSTNAME])
-                if client[C_SESSION_HANDLE] == '':
-                    if self.ltsConf.sectionExists(client[C_MAC]):
-                        self.savedClientReset(client)
-                    else:
-                        self.cstore.remove(client.iter)
-                else:
-                    client[C_SYSTEM_HANDLE] = ''
+        print "Disconnect from", handle
+        
+        def determine_offline(client):
+            if client.hsystem == '' and client.users == {}:
+                client.set_offline()
+        
+        for client in structs.clients:
+            if client.hsystem == handle:
+                if self.getSelectedGroup()[1].has_client(client) or self.isDefaultGroupSelected():
+                    shutdownNotify(client.get_name())
+                client.hsystem = ''
+                determine_offline(client)
                 break
-            elif client[C_SESSION_HANDLE] == handle:
-                logoutNotify(client[C_USER], client[C_HOSTNAME])
-                if client[C_SYSTEM_HANDLE] == '':
-                    if self.ltsConf.sectionExists(client[C_MAC]):
-                        self.savedClientReset(client)
-                    else:
-                        self.cstore.remove(client.iter)
-                else:
-                    client[C_USER] = ''
-                    client[C_SESSION_HANDLE] = ''
-                    type = client[C_TYPE]
-                    client[C_PIXBUF] = self.imagetypes[type]
+            
+            elif handle in client.users:
+                if self.getSelectedGroup()[1].has_client(client) or self.isDefaultGroupSelected():
+                    logoutNotify(client.users[handle], client.get_name())
+                del client.users[handle]
+                determine_offline(client)
                 break
-        self.refresh()
+        self.fillIconView(self.getSelectedGroup()[1])
     
-
-    def savedClientReset(self, client):
-        client[C_SESSION_HANDLE] = ''
-        client[C_SYSTEM_HANDLE] = ''
-        client[C_USER] = ''
-        client[C_TYPE] = 'offline'
-        client[C_PIXBUF] = self.offline
-
     def amp_gotClients(self, handles):
+        print "Got clients:", ', '.join(handles) or 'None'
         for handle in handles:
-            d = self.daemon.command(handle,  u"""
+            d = self.daemon.command(handle, u"""
                 VERSION=$(dpkg-query -W -f '${Version}' epoptes-client 2>/dev/null)
                 VERSION=${VERSION:-0.1}
                 echo "$USER\n$HOSTNAME\n$IP\n$MAC\n$TYPE\n$UID\n$VERSION\n$$"
@@ -439,33 +445,76 @@ export $(tr '\\0' '\\n' < /proc/$p/environ | egrep '^DISPLAY=|^XAUTHORITY=')
     
     def on_button_close_clicked(self, widget):
         self.get('warningDialog').hide()
-
     
-    # FIXME: Proofread this
+    def on_group_selection_changed(self, treeselection):
+        self.cstore.clear()
+        selected = self.getSelectedGroup()
+                
+        if selected is not None:
+            self.fillIconView(selected[1])
+        else:
+            if not self.default_group.ref.valid():
+                return
+            self.gtree.get_selection().select_path(self.default_group.ref.get_path())
+        self.get('remove_group').set_sensitive(not self.isDefaultGroupSelected())
+        self.set_move_group_sensitivity()
+    
+    def addToIconView(self, client):
+        """Properly add a Client class instance to the clients iconview."""
+        # If there are one or more users on client, add a new iconview entry
+        # for each one of them.
+        if client.users:
+            for hsession, user in client.users.iteritems():
+                self.cstore.append([self.calculateLabel(client, user), self.imagetypes[client.type], client, hsession])
+                self.getScreenshots(hsession, self.getSelectedGroup()[1])
+        else:
+            self.cstore.append([self.calculateLabel(client), self.imagetypes[client.type], client, ''])
+    
+    def fillIconView(self, group):
+        """Fill the clients iconview from a Group class instance."""
+        self.cstore.clear()
+        if self.isDefaultGroupSelected():
+            clients_list = [client for client in structs.clients if client.type != 'offline']
+        else:
+            clients_list = group.get_members()
+        # Add the new clients to the iconview
+        for client in clients_list:
+            self.addToIconView(client)
+    
+    def isDefaultGroupSelected(self):
+        """Return True if the default group is selected"""
+        return self.getSelectedGroup()[1] is self.default_group
+    
+    def getSelectedGroup(self):
+        """Return a 2-tuple containing the iter and the instance
+        for the currently selected group."""
+        iter = self.gtree.get_selection().get_selected()[1]
+        if iter:
+            return (iter, self.gstore[iter][G_INSTANCE])
+        else:
+            return None
+        
     def addClient(self, handle, r, already=False):
         # already is True if the client was started before epoptes
         user, host, ip, mac, type, uid, version, pid = r.strip().split()
-
-        # If the user logged in the client is the same as
-        # the UI user, don't add the client to the iconview.
-        if user == os.getenv('USER'):
-            # But verify that the sysadmin isn't using the same username in all
-            # standalone clients.
-            try:
-                for line in open('/proc/' + pid + '/environ'):
-                    if 'epoptes-client' in line:
-                        return
-            except:
-                pass
-        # Do the same if there is a hostname filter specified as a parameter
-        if not _startswith(host, self.host_filter):
-            return
-            
+        print "---\n**addClient's been called for", handle
+        
+        # Check if the incoming client is the same with the computer in which
+        # epoptes is running, so we don't add it to the list.
+        # FIXME FiXME: Both ifs don't work for root clients that run in the same
+        # computer as epoptes
+        if mac in self.current_macs:
+            print "* Won't add this client to my lists"
+            return False
+        
+        print '  Continuing inside addClient...'
+        
         # Compatibility check
         if [int(x) for x in re.split('[^0-9]*', version)] < COMPATIBILITY_VERSION:
             if not self.shownCompatibilityWarning:
                 self.shownCompatibilityWarning = True
                 dlg = self.get('warningDialog')
+                # Show different messages for LTSP clients and standalones.
                 msg = _("There was a try to connect an epoptes-client with version %s \
 which is incompatible with the current epoptes version.\
 \n\nYou should update your chroot.") % version
@@ -473,68 +522,81 @@ which is incompatible with the current epoptes version.\
                 dlg.set_transient_for(self.mainwin)
                 dlg.show()
             self.daemon.command(handle, u"exit")
-            return
+            return False
         
-        
-        mac = mac.upper()
-        # Check if the client already exists
-        index = None
-        for i in range(len(self.cstore)):
-            if self.cstore[i][C_IP] == ip or \
-                (self.cstore[i][C_MAC] == mac and self.cstore[i][C_IP] == '') or \
-                (self.cstore[i][C_HOSTNAME] == host and \
-                self.cstore[i][C_IP] == '' and type == 'thin' and uid != 0):
-                index = i
-                entry = self.cstore[i]
+        client = None
+        for inst in structs.clients:
+            # Find if the new handle is a known client
+            if mac == inst.mac:
+                client = inst
+                print '* This is an existing client'
                 break
-
-        # If it doesn't exist, create a new empty entry
-        if index is None:
-            entry = ['']*9
-
-        
-        # Check if the new client is a root epoptes-client instance or not
+        if client is None:
+            print '* This client is a new one, creating an instance'
+            client = structs.Client(mac=mac)
+            
+        # Update/fill the client information
+        client.type, client.hostname = type, host
         if int(uid) == 0:
-            entry = [host, mac, entry[C_SESSION_HANDLE], handle, type, 
-                entry[C_USER], self.imagetypes[type], ip, '']
+            # This is a root epoptes-client
+            print '* I am a root client'
+            client.hsystem = handle
         else:
-            # For standalone clients, use mac etc if not provided already
-            entry = [host, entry[C_MAC], handle, entry[C_SYSTEM_HANDLE], type, 
-                user, self.imagetypes[type], ip, '']
-            # Pop up a proper graphical notification (python-notify)
-            if already:
-                loggedinNotify(user, host)
-            else:
+            # This is a user epoptes-client
+            print '* I am a user client, will add', user, 'in my list'
+            client.add_user(user, handle)
+            if not already and (self.getSelectedGroup()[1].has_client(client) or self.isDefaultGroupSelected()):
                 loginNotify(user, host)
         
-        # If there is already an entry for this client, just update it
-        # else treat this client as a new entry
-        if index is not None:
-            self.cstore[index] = entry
+        self.fillIconView(self.getSelectedGroup()[1])
+    
+    def setLabel(self, row):
+        inst = row[C_INSTANCE]
+        if row[C_SESSION_HANDLE]:            
+            user = row[C_INSTANCE].users[row[C_SESSION_HANDLE]]
         else:
-            self.cstore.append(entry)
-
-        if int(uid) != 0:
-            self.getScreenshots(entry[C_SESSION_HANDLE])
+            user = ''
+        row[C_LABEL] = self.calculateLabel(inst, user)
+    
+    def calculateLabel(self, client, username=''):
+        """Return the iconview label from a hostname/alias
+        and a username, according to the user options.
+        """
+        user_pos, name_pos = self.cView_order
         
-        self.refresh()
-
+        alias = client.get_name()
+        if username == '' or user_pos == -1:
+            return alias
+        else:
+            if user_pos == 0:
+                label = username
+                if name_pos == 1:
+                    label += " (%s)" % alias
+            elif name_pos == 0:
+                label = alias
+                if user_pos == 1:
+                    label += " (%s)" % username
+            return label
+    
     def getAllScreenshots(self):
         # TODO: Ask for screenshots for every client (Look diff at Rev:326)
         pass
 
-
-    def getScreenshots(self, handle):
+    def getScreenshots(self, handle, group):
+        # TODO: Implement this using gtk.TreeRowReferences instead
+        # of searching the whole model (Need to modify execOnClients)
+        if not group is self.getSelectedGroup()[1]:
+            return False
         for client in self.cstore:
             if handle == client[C_SESSION_HANDLE]:
                 self.execOnClients(self.c.SCREENSHOT
-                    % (self.scrWidth, self.scrHeight), handles=[handle], 
+                    % (self.scrWidth, self.scrHeight), handles=[handle],
                         reply=self.updateScreenshots)
-                return
+                return False
 
 
     def updateScreenshots(self, handle, reply):
-        if reply is None:
+        if not reply:
             return
         try:
             rowstride, size, pixels = reply.strip().split('\n', 2)
@@ -544,8 +606,8 @@ which is incompatible with the current epoptes version.\
         width, height = size.split('x')
         for i in self.cstore:
             if handle == i[C_SESSION_HANDLE]:
-                pxb = gtk.gdk.pixbuf_new_from_data(pixels, 
-                    gtk.gdk.COLORSPACE_RGB, False, 8, int(width), int(height), 
+                pxb = gtk.gdk.pixbuf_new_from_data(pixels,
+                    gtk.gdk.COLORSPACE_RGB, False, 8, int(width), int(height),
                     rowstride)
                 self.cstore[i.path][C_PIXBUF] = pxb
                 # We want to ask for thumbnails every 5 sec after the last one.
@@ -553,47 +615,16 @@ which is incompatible with the current epoptes version.\
                 # send a thumbnail, we'll ask for one every 12 secs.
                 # TODO: check if there are cases where we want to continue
                 # asking for screenshots even if we got an empty/broken one.
-                gobject.timeout_add(5000, self.getScreenshots, handle)
+                gobject.timeout_add(5000, self.getScreenshots, handle, self.getSelectedGroup()[1])
                 break
     
-    # FIXME: Just shut up and fix me
-    def addStockImage(self, name, filename):
-        pixbuf = gtk.gdk.pixbuf_new_from_file(filename)#_at_size(filename,16,16)
-        iconset = gtk.IconSet(pixbuf)
-
-        factory = gtk.IconFactory()
-        factory.add(name, iconset)
-        factory.add_default()
-    
-    def loadClients(self):
-        macs = self.ltsConf.getSavedClients()#Sections()
-        for mac in macs:
-            mac = mac.upper()
-            exists = False
-            for client in self.cstore:
-                if client[C_MAC] == mac:
-                    exists = True
-                    break
-            if not exists:
-                hostname = self.ltsConf.getItem(mac, 'HOSTNAME')
-                if _startswith(hostname, self.host_filter):
-                    self.cstore.append([hostname, mac, '', '', 'offline', '', 
-                                         self.offline, '', hostname])
-    
-    
     def getSelectedClients(self):
         selected = self.cview.get_selected_items()
         items = []
         for i in selected:
-            path = self.cfilter.convert_path_to_child_path(
-                self.csort.convert_path_to_child_path(i[0]))
-            items.append(self.cstore[path])
+            items.append(self.cstore[i])
         return items
-
-    
-    def setVisibleClients(self, model, iter):
-        return True #FIXME: add a gtk.Entry to work as a filter and implement this function
-    
+        
     def changeHostname(self, mac, new_name):
         pass #FIXME: Implement this (virtual hostname)
     
@@ -601,7 +632,7 @@ which is incompatible with the current epoptes version.\
         subprocess.Popen(["xdg-open", link])
 
     def openHelpLink(self, widget):
-        self.openLink("http://epoptes.sourceforge.net")
+        self.openLink("http://www.epoptes.org")
 
     def openBugsLink(self, widget):
         self.openLink("https://bugs.launchpad.net/epoptes")
@@ -609,22 +640,21 @@ which is incompatible with the current epoptes version.\
     def openQuestionsLink(self, widget):
         self.openLink("https://answers.launchpad.net/epoptes")
 
-    ## TODO: fix the links, and replace the forum link with a translations link
-    def openForumLink(self, widget):
-        self.openLink("http://alkisg.mysch.gr/steki/index.php?board=67.0")
+    def openTranslationsLink(self, widget):
+        self.openLink("http://www.epoptes.org/translations")
 
     def openIRCLink(self, widget):
         user = os.getenv("USER")
         if user is None:
-            user = "teacher."
-        self.openLink("http://webchat.freenode.net/?nick=" + user +
-            "&channels=linux.sch.gr&prompt=1")
+            user = "epoptes_user." # The dot is converted to a random digit
+        self.openLink("http://webchat.freenode.net/?nick=" + user + 
+            "&channels=ltsp&prompt=1")
     
     ## FIXME: We don't use this (we want to). there was a problem with twisted :-\
     def iconsSizeScaleChanged(self, widget):
         adj = self.get('iconsSizeAdjustment')
         self.scrWidth = int(adj.get_value())
-        self.scrHeight = int(3*self.scrWidth/4) # Κeep the 4:3 aspect ratio
+        self.scrHeight = int(3 * self.scrWidth / 4) # Κeep the 4:3 aspect ratio
         self.getAllScreenshots()
 
     def scrIncreaseSize(self, widget):
@@ -640,7 +670,7 @@ which is incompatible with the current epoptes version.\
     ## END_FIXME
 
     def contextMenuPopup(self, widget, event):
-        clicked = widget.get_path_at_pos(int(event.x),int(event.y))
+        clicked = widget.get_path_at_pos(int(event.x), int(event.y))
 
         if event.button == 3:
             if widget is self.cview:
@@ -662,35 +692,22 @@ which is incompatible with the current epoptes version.\
             if widget is self.cview:
                 menu = self.get('clients').get_submenu()
             
-            menu.popup(None,None,None,event.button,event.time)
+            menu.popup(None, None, None, event.button, event.time)
             menu.show()
             return True
 
-    def setClientMenuSensitivity(self, widget=None):
+    def on_clients_selection_changed(self, widget=None):
         selected = self.getSelectedClients()
         sensitive = False
         if len(selected) == 1:
             sensitive = True
         self.get('miClientProperties').set_sensitive(sensitive)
         self.get('tb_client_properties').set_sensitive(sensitive)
-
-    
-    ## FIXME: This is not yet finished, implement it
-    def editClientName(self, widget):
-
-        dlg = self.get('entrydialog')
-        entry = self.get('newhostname')
-        if widget is self.get('change_name'):
-            hostname = self.get('client_name').get_text()
-            mac = self.get('client_mac').get_text()
-            entry.set_text(hostname)
-        resp = dlg.run()
-        dlg.hide()
-        if resp == 1:
-            self.refresh()
-            print "No changes made, not yet implemented"
-            
-    ## END_FIXME
+        
+        if len(selected) >= 1:
+            self.get('miRemoveFromGroup').set_sensitive(True)
+        else:
+            self.get('miRemoveFromGroup').set_sensitive(False)
 
     
     ## FIXME / FIXUS: Proofread this (root etc...)
@@ -701,7 +718,7 @@ which is incompatible with the current epoptes version.\
         if len(self.cstore) == 0:
             # print 'No clients'
             return False
-        if (clients !=[] or handles !=[]) and warning !='':
+        if (clients != [] or handles != []) and warning != '':
             if self.warnDlgPopup(warning) == False:
                 return
         if clients == [] and handles != []:
@@ -713,8 +730,8 @@ which is incompatible with the current epoptes version.\
         for client in clients:
             if (root == "auto" or not root) and client[C_SESSION_HANDLE] != '':
                 handle = client[C_SESSION_HANDLE]
-            elif root and client[C_SYSTEM_HANDLE] != '':
-                handle = client[C_SYSTEM_HANDLE]
+            elif root and client[C_INSTANCE].hsystem != '':
+                handle = client[C_INSTANCE].hsystem
             else:
                 continue
             cmd = self.daemon.command(handle, unicode(command))
@@ -727,7 +744,7 @@ which is incompatible with the current epoptes version.\
         if len(clients) == 0: # No client selected, send the command to all
             clients = self.cstore
         else: # Show the warning only when no clients are selected
-            warn=''
+            warn = ''
         self.execOnClients(command, clients, reply, root, warning=warn)
     
     # END_FIXUS
@@ -758,31 +775,32 @@ which is incompatible with the current epoptes version.\
             clients = self.cstore
 
         if as_root:
-            screen_params="bash -l"
+            screen_params = "bash -l"
         else:
-            screen_params="-l"
+            screen_params = "-l"
 
         for client in clients:
-            if client[C_TYPE] == 'offline':
+            inst = client[C_INSTANCE]
+            if inst.type == 'offline':
                 continue
-            elif client[C_TYPE] == 'thin' and not as_root:
-                server="127.0.0.1"
+            elif inst.type == 'thin' and not as_root:
+                server = "127.0.0.1"
             else:
-                server="server"
+                server = "server"
 
             port = self.findUnusedPort()
 
-            subprocess.Popen(['xterm', '-e', 'socat', 
+            subprocess.Popen(['xterm', '-e', 'socat',
                 'tcp-listen:%d,keepalive=1' % port, 'stdio,raw,echo=0'])
-            self.execOnClients(("""./execute sh -c "cd; sleep 1; """ +
-                """TERM=xterm exec socat SYSTEM:'exec screen %s',pty,""" +
+            self.execOnClients(("""./execute sh -c "cd; sleep 1; """ + 
+                """TERM=xterm exec socat SYSTEM:'exec screen %s',pty,""" + 
                 """stderr tcp:$SERVER:%d" """) % (screen_params, port),
                 [client], root=as_root)
 
     def execInTerminal(self, widget, command):
         name = widget.get_child().get_text()
         subprocess.Popen([ 'x-terminal-emulator', '-e', 'sh', '-c', command
-            + ' && read -p "Script \'%s\' finished succesfully. Press [Enter] to close this window." dummy' %name
-            + ' || read -p "Script \'%s\' finished with errors. Press [Enter] to close this window." dummy' %name]
+            + ' && read -p "Script \'%s\' finished succesfully. Press [Enter] to close this window." dummy' % name
+            + ' || read -p "Script \'%s\' finished with errors. Press [Enter] to close this window." dummy' % name]
             )
 
